@@ -253,9 +253,9 @@ def test_widget_notes_remove_artf_evts(qtbot):
     w.notes.add_event('Artefact', (1, 2), 'EEG Pz-Oz (scalp)')
     w.notes.add_event('Artefact', (3, 6), 'EEG Pz-Oz (scalp)')
     w.notes.add_event('Artefact', (7, 10), 'EEG Pz-Oz (scalp)')
-    w.notes.add_event('Artefact', (14, 31), 'EEG Pz-Oz (scalp)')
+    w.notes.add_event('Artefact', (14, 31), '')
     w.notes.add_event('Artefact', (70, 85), 'EEG Pz-Oz (scalp)')
-    w.notes.add_event('Artefact', (87, 90), 'EEG Pz-Oz (scalp)')
+    w.notes.add_event('Artefact', (87, 90), '')
     w.notes.add_event('Artefact', (90, 92), 'EEG Pz-Oz (scalp)')
     w.notes.add_event('Artefact', (105, 120), 'EEG Pz-Oz (scalp)')
     w.notes.add_event('Artefact', (125.0, 125.2), 'EEG Pz-Oz (scalp)')
